// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "asset/loader/JSONLoader.hpp"

namespace Dawn {
  class Game;

  class LocaleManager {
    private:
      std::weak_ptr<Game> game;
      std::shared_ptr<JSONLoader> languageAsset;

    public:
      /**
       * Creates a new LocaleManager.
       */
      LocaleManager();

      /**
       * Initializes the LocaleManager.
       * 
       * @param game The game to initialize with.
       */
      void init(const std::shared_ptr<Game> &game);

      /**
       * Returns a string from the locale.
       * 
       * @param key The key of the string.
       * @param replacements The replacements to make in the string.
       * @return The string.
       */
      std::string getString(
        const std::string &key,
        const std::unordered_map<std::string, std::string> replacements =
        std::unordered_map<std::string, std::string>()
      );

      /**
       * Destroys the LocaleManager.
       */
      ~LocaleManager();
  };
}