// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/IRenderHost.hpp"
#include "display/BackBufferRenderTarget.hpp"
#include <glad/glad.h>
#include <GLFW/glfw3.h>

#define DAWN_GLFW_WINDOW_WIDTH_DEFAULT 1280
#define DAWN_GLFW_WINDOW_HEIGHT_DEFAULT 720

namespace Dawn {
  class Game;

  class RenderHost : public IRenderHost {
    public:
      std::shared_ptr<BackBufferRenderTarget> backBufferRenderTarget;
      GLFWwindow *window = nullptr;

      /**
       * Initializes the GLFW RenderHost.
       */
      RenderHost();

      void init(const std::shared_ptr<Game> game) override;
      void update(const std::shared_ptr<Game> game) override;
      bool_t isCloseRequested() override;
      std::shared_ptr<RenderTarget> getBackBufferRenderTarget() override;

      ~RenderHost();
  };
}