
// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "scene/SceneList.hpp"
#include "component/display/Camera.hpp"
#include "prefab/SimpleSpinningCube.hpp"
#include "component/display/material/SimpleTexturedMaterial.hpp"
#include "display/mesh/QuadMesh.hpp"
#include "component/ui/UICanvas.hpp"

using namespace Dawn;

void Dawn::helloWorldScene(Scene &s) {
  // while(!s.getGame()->assetManager.isLoaded(font)) {
  //   s.getGame()->assetManager.update();
  // }

  auto cameraItem = s.createSceneItem();
  auto camera = cameraItem->addComponent<Camera>();
  cameraItem->lookAt({ 3, 3, 3 }, { 0, 0, 0 }, { 0, 1, 0 });
  camera->clipFar = 99999.99f;

  glm::vec2 position = { 0, 0 };
  glm::vec2 size = { 1, 1 };
  auto quad = s.createSceneItem();
  auto quadMesh = std::make_shared<Mesh>();
  quadMesh->createBuffers(QUAD_VERTICE_COUNT, QUAD_INDICE_COUNT);
  QuadMesh::buffer(
    quadMesh,
    glm::vec4(position.x, position.y, size.x, size.y),
    glm::vec4(0, 0, 1, 1),
    0, 0, 0
  );

  auto quadRenderer = quad->addComponent<MeshRenderer>();
  quadRenderer->mesh = quadMesh;

  auto quadMaterial = quad->addComponent<SimpleTexturedMaterial>();
  quadMaterial->setColor(COLOR_WHITE);

  auto uiCanvasItem = s.createSceneItem();
  auto uiCanvas = uiCanvasItem->addComponent<UICanvas>();
}